#delimit ;
cap log close;
set more off; 
global directory /disk/agedisk3/oebb.work/cbehrer/oregon/ ;
local c_date = c(current_date);
local cdate = subinstr("`c_date'", " ", "_", .);
local ctime = c(current_time);
log using $directory/log/sampleselection/0sampleselection_PF_`cdate'.log, replace;

* Project: 		Oregon Health Plan Choice Analysis											;
* Program: 		2_0sampleselection Perfect Forecast											;
* Author:		Chris Behrer (christopher_behrer@post.harvard.edu) 							;
* Dates:		Created :	11/09/2015														;
*				Modified: 	09/21/2016														;

* Program purpose 																			;
* Create a list of beneficiaries who meet eligibility requirements in each year				;
* Tabulate some sanity checks on the fraction of these people with claims 					;
* Sample requirements																		;
*		0) Start with unique individuals and families in the eligibility file 				;
* 		1) Require covereage (some values of the plan variable indicate declined covered) 	; 
* 			1.5) Count family size AND flag double covered individuals here					; 
* 				 We need to do this before we start dropping observations to make sure that ; 
*				 We accurately identify double covered (COBRA is still coverage) AND to make;
* 				 sure that if we drop any individual covered in a family, that we drop the  ; 
*				 entire family, because their choice will not be accurately modeled without ; 
* 				 the claims of all family members 											;
*		2) Require individuals to not be covered by COBRA or be self-pay retirees 			; 
* 		3) Require individuals to be continuously eligible, for 2008, for the full year		; 
* 		   and for years beyond 2008 for the full year and full prior year					; 
*		4) Require that we have premium AND district contribution data						; 
*		5) Require that individuals be covered by only one plan								; 
* 		6) Require that individuals only have claims in one plan							; 
*		7) Drop individuals in masked entities												;	 

* 0. Pass globals, set log 	;
local macros $directory/do/macros/ ;
local in_data $directory/data/given/clean ;

clear all;
* Create datafile locals ;
local elig_data "$directory/data/cleaning/2deduplicated/deduplicated_eligibility_11_Jun_2015.dta" ; 
local facility "$directory/data/cleaning/3devoid/facilityNEW_devoid_04_Mar_2016.dta" ; 
local professional "$directory/data/cleaning/3devoid/professional_devoid_04_Mar_2016.dta" ;
local drugs "$directory/data/cleaning/3devoid/drugs_devoid__4_Mar_2016.dta" ; 
local out_data $directory/data/built/sample/ ;
local choiceset "$directory/data/clean/choiceset_29_Mar_2016.dta" ;
local premiumdata "$directory/data/built/premium/finalpremium_29_Mar_2016.dta" ;

foreach year in 2008 2009 2010 2011 2012 2013 { ;
* Read in eligibility data ; 
use `elig_data', clear ; 
	keep age_month age_yr entitynum tiercd memid empid eligmo_date eligflag empstatus statusgrp familyid personidIE personid plan drugplan dentcov drugcov hearcov medcov mhsacov visioncov sex plangrp relat ; 

	gen planyear = 2008 if eligmo_date >= mdy(10,1,2008) & eligmo_date <=mdy(9,30,2009) ;
	replace planyear = 2009 if eligmo_date >= mdy(10,1,2009) & eligmo_date <=mdy(9,30,2010) ;
	replace planyear = 2010 if eligmo_date >= mdy(10,1,2010) & eligmo_date <=mdy(9,30,2011) ;
	replace planyear = 2011 if eligmo_date >= mdy(10,1,2011) & eligmo_date <=mdy(9,30,2012) ;  
	replace planyear = 2012 if eligmo_date >= mdy(10,1,2012) & eligmo_date <=mdy(9,30,2013) ;  
	replace planyear = 2013 if eligmo_date >= mdy(10,1,2013) & eligmo_date <=mdy(9,30,2014) ; 
	replace planyear = 2014 if eligmo_date >= mdy(10,1,2014) & eligmo_date <=mdy(9,30,2015) ; 
	
	* For PF we want to keep new beneficiaries, so do not require prior year eligibility ; 
	keep if planyear == `year' ; 

	* Unmask entities ;
	* Some entities had such few people that the names were masked for confidentiality ; 
	* We have a crosswalk to unmask some of these entities to get the proper choice sets and match premium data ; 
	* there are 4 entitynum values that have letters in them, but unmask.do treats entitynum numerically ;
	* we don't have information to include these 4 entities in our analysis, so drop them  ;
	drop if substr(entitynum,1,4) == "S041";
	drop if substr(entitynum,1,4) == "S399";
	drop if substr(entitynum,1,4) == "U017";
	drop if substr(entitynum,1,4) == "U018";
	replace entitynum = "0020" if entitynum == "00020"; 
	replace entitynum = "0020" if entitynum == "20"; 
	
	destring entitynum, replace;
	quietly do `macros'/unmask.do; 
	tostring entitynum, replace;	
	
* Begin sample selection 									; 
* 0 Unique individuals and families in the eligibility file ; 
	preserve ; 
		keep if planyear == `year' ; 
		keep personid familyid ; 
		duplicates drop ; 
		isid(personid) ;
		gen count_individuals = _N ;
		drop personid  ; 
		duplicates drop ; 
		isid(familyid) ;
		gen count_family = _N ; 
		drop familyid ;
		duplicates drop ; 
		assert _n == 1 ; 
		gen step = 0 ; 
		gen description = "Number of unique individuals and families in the eligibility file" ; 
		order step description count_individuals count_family ;  
		list ; 
		tempfile step0 ; 
		save `step0';
	restore ; 
 	
	* creating count0 here is a test, what does double coverage look ; 
	* like if we count observations with declined coverage? ; 
	* I do not use this count to select, but just investigate ; 
	* people that would be dropped because of double covereage counting these observation ; 
	* but are not dropped below are those who declined coverage under one policy holder ; 
	* and have it under another, so they are not really double covered and shouldn't be dropped ; 
	gen subscriber=substr(personid, 1, 9);
	assert subscriber==familyid if personid!="";
	gen member=substr(personid, 15, 23);
	cap drop count;
	preserve ;
		keep subscriber member personid;
		drop if personid=="";
		duplicates drop;
		bys member: gen count0=_N;
		keep count0 personid;
		
		tempfile count0;
		save `count0';
	restore ;
	merge m:1 personid using `count0';
	assert _merge==3;
	drop _merge;
	tab count0;

	replace plan = trim(plan) ; 
	replace plan = substr(plan,1,5) ;
	
* 1 Drop observations indicating declined coverage ; 
	drop if substr(plan,1,3) == "OPT" ; 
	drop if substr(plan,1,2) == "NE" ; 
	drop if substr(plan,1,1) == "~" ; 
	preserve ; 
		keep if planyear == `year' ; 
		keep personid familyid ; 
		duplicates drop ; 
		gen count_individuals = _N ;
		isid(personid) ;
		drop personid ; 
		duplicates drop ;
		isid(familyid) ;		
		gen count_family = _N ; 
		drop familyid ;
		duplicates drop ; 
		assert _n == 1 ; 
		gen step = 1 ; 
		gen description = "Number of unique individuals and families with coverage" ; 
		order step description count_individuals count_family ; 
		list ; 
		tempfile step1 ; 
		save `step1'; 
	restore ; 

* 1.5 Calculate family size and double coverage ; 
	preserve ; 
	* we need to tabulated each family's size here, so that we can drop families that lose any individuals throughout the rest of this process ; 
	keep if planyear == `year' ;
	keep personid familyid ; 
	duplicates drop ; 
	isid (personid);
	duplicates drop ; 
	gen counter = 1 ; 
	egen famsize = sum(counter), by(familyid) ; 
	drop personid ; 
	duplicates drop ; 
	isid(familyid) ; 
	tempfile famsize ; 
	save `famsize' ; 
	restore ; 		
	
	* ID beneficiaries (and the families covering them) if the beneficiary is covered by multiple subscribers and/or multiple plans ;
	drop subscriber member ; 
	gen subscriber=substr(personid, 1, 9);
	assert subscriber==familyid if personid!=""; 
	gen member=substr(personid, 15, 23);
	preserve ;
		keep subscriber member personid;
		drop if personid=="";
		duplicates drop;
		bys member: gen count1=_N;
		keep count1 personid;
		tempfile count1;
		save `count1';
	restore ;
	
	merge m:1 personid using `count1';
	assert _merge==3;
	drop _merge;
	tab count1;
	tab count0 count1 ; 

	* count1 > 1 here indicates that they are still double covered after dropping months that indicate no coverage ; 
	* so these individuals should definitely be dropped ; 
	* we also need to drop the entire family, if there is a double covered child, then both parents need to be dropped ; 
	* we do this before dropping COBRA/retirees because those statuses still reflect having coverage: an individual could be ; 
	* double covered, once by a standard employee and once by a COBRA/retiree beneficiaries ; 
	* we drop cobra/retiree because we do not observe entity contibutions, but they still definitely have coverage, so double coverage still applies ; 
	egen maxcount1 = max(count1), by(familyid) ;
	tab count0 maxcount1 ; 
	* we don't drop here so that we can have the same first 7 rows in sample selection ; 
	* create an indicator for double coverage so that we can drop later ; 
	* we have to make this indicator now before we start dropping observations for other reasons ; 
	* in case someone is double covered with one policy holder in a masked entity or something like that ; 
	gen doublecov = . ; 
	replace doublecov = 1 if maxcount1 > 1 ; 
	tab count0 count1 ; 
	* count1 == 1 but count0 > 1 are more unclear ;
	* I think that we should keep them. These are members who have opted out of coverage under one family, and have coverage under another ; 
	* they are thus not double covered ; 

	* Now check multiple plans ; 
	* members should only be covered by 1 plan ; 
	preserve ;
		keep member plan;
		drop if member=="";
		duplicates drop;
		bys member: gen count2=_N;
		keep count2 member;
		duplicates drop ; 
		tempfile count2;
		save `count2';
	restore ;
	merge m:1 member using `count2';
	assert _merge == 3 ; 
	drop _merge ; 
	gen multiplan = . ;
	replace multiplan = 1 if count2 > 1 ; 
 
	* families should only have 1 plan ; 
	preserve ;
		keep familyid plan;
		drop if familyid=="";
		duplicates drop;
		bys familyid: gen count3=_N;
		keep count3 familyid;
		duplicates drop ; 
		tempfile count3;
		save `count3';
	restore ;
	merge m:1 familyid using `count3';
	assert _merge == 3 ; 
	drop _merge ; 
	assert count3 != . ; 
	replace multiplan = 1 if count3 > 1 ; 
	tab count2 count3 ; 
	replace multiplan = 0 if multiplan == . ;
	egen max_multi = max(multiplan), by(familyid) ; 
	tab multiplan max_multi ; 
	assert max_multi != . ; 
	replace doublecov = 1 if max_multi > 0 ; 

	drop max_multi multiplan maxcount1 ; 
	* in addition to checking double coverage in the eligibility file, we need to check claims ; 
	* if a member has claims from either multiple plans or multiple subscribers then we need to drop the entire set of families covering that member ; 
	preserve ; 
	keep member ; 
	duplicates drop ; 
	tempfile members ; 
	save `members' ; 
	restore ; 
	
	* make sure that they do not have claims from multiple plans or subscribers  ; 
	preserve ; 

	use `drugs', clear ; 
	gen srvcdt2 = date(srvcdt, "MDY") ;
	drop srvcdt ; 
	rename srvcdt2 srvcdt ; 
	replace plan = trim(plan) ;
	replace plan = substr(plan,1,5) ;
	drop if allowedamt < 0 ; 
	drop if dayssupply < 0 ; 
	
	gen 	planyear = 2008 if srvcdt >= mdy(10,1,2008) & srvcdt <= mdy(9,30,2009) ;
	replace planyear = 2009 if srvcdt >= mdy(10,1,2009) & srvcdt <= mdy(9,30,2010) ;
	replace planyear = 2010 if srvcdt >= mdy(10,1,2010) & srvcdt <= mdy(9,30,2011) ;
	replace planyear = 2011 if srvcdt >= mdy(10,1,2011) & srvcdt <= mdy(9,30,2012) ; 
	replace planyear = 2012 if srvcdt >= mdy(10,1,2012) & srvcdt <= mdy(9,30,2013) ; 
	replace planyear = 2013 if srvcdt >= mdy(10,1,2013) & srvcdt <= mdy(9,30,2014) ; 
	replace planyear = 2014 if srvcdt >= mdy(10,1,2014) & srvcdt <= mdy(9,30,2015) ; 
	replace planyear = 2007 if srvcdt < mdy(10,1,2008) ; 
	drop if planyear == 2007 ; 
	tab planyear;
	assert planyear!=. ;
	keep if  planyear == `year';
	keep personid familyid plan ; 
	tempfile drug0 ; 
	save `drug0' ; 
	
	use `facility', clear ; 
	drop proc_surg ; 
	append using `professional' ; 
	drop proc_surg ; 
	drop if allowedamt < 0 ;
	keep if covtypcd == 1;
	drop covtypcd ; 	
	gen srvcdt2=date(firstsrvcdt, "MDY");
	format srvcdt2 %d;	
	gen 	planyear = 2008 if srvcdt2 >= mdy(10,1,2008) & srvcdt2 <= mdy(9,30,2009) ;
	replace planyear = 2009 if srvcdt2 >= mdy(10,1,2009) & srvcdt2 <= mdy(9,30,2010) ;
	replace planyear = 2010 if srvcdt2 >= mdy(10,1,2010) & srvcdt2 <= mdy(9,30,2011) ;
	replace planyear = 2011 if srvcdt2 >= mdy(10,1,2011) & srvcdt2 <= mdy(9,30,2012) ; 
	replace planyear = 2012 if srvcdt2 >= mdy(10,1,2012) & srvcdt2 <= mdy(9,30,2013) ; 
	replace planyear = 2013 if srvcdt2 >= mdy(10,1,2013) & srvcdt2 <= mdy(9,30,2014) ; 
	replace planyear = 2014 if srvcdt2 >= mdy(10,1,2014) & srvcdt2 <= mdy(9,30,2015) ; 
	replace planyear = 2007 if srvcdt2 < mdy(10,1,2008) ; 
	drop if planyear == 2007 ; 
	tab planyear;
	drop srvcdt2 ; 
	assert planyear!=. ;
	keep if  planyear == `year';
	keep personid familyid plan ; 
	tempfile med0 ; 
	save `med0' ; 
	append using `drug0' ; 
	duplicates drop ; 
	gen member=substr(personid, 15, 23) ; 
	merge m:1 member using `members' ; 
 
	keep if _merge == 3 ; 
	drop _merge ; 

	unique(member plan) ;
	unique(member familyid) ;
	unique(member) ;
	*** these should return the same counts - individuals should only have claims in 1 plan and 1 family; 
	*** this isn't universally true, we need to drop all observations for people with multiple plans ; 
	*** then merge in the plan the individual is recorded a having chosen in the eligibility file and keep only those claims ; 
	
	bys member : gen count4=_N ;
	* this identifies members with claims either in two or more different plans or in two or more different families ; 
	* both need to be dropped ; 
	
	* we will drop these count4 > 1 observations ; 
	* we also need the plan for the count4 == 1 observations to drop if not chosen plan ; 
	* and we need 1 observation per member ; 
	* so make all plans the same for count4 > 1 so there is just 1 observation for these members ; 
	assert count4 !=. ; 
	replace plan = "DUPS" if count4 > 1 ; 
	keep member plan count4 ; 
	rename plan claimsplan ;
	duplicates drop ; 
	isid (member) ;
	tempfile claims ; 
	save `claims' ; 
	restore ; 

	merge m:1 member using `claims' ; 
	replace claimsplan = plan if _merge == 1; 
	replace count4 = 1 if _merge == 1 ; 
	tab _merge ; 
	drop _merge ; 
	assert claimsplan != "" ; 
	assert plan != "" ; 
	assert count4 != . ; 

	gen wrongplan = 1 if plan != claimsplan ; 
	replace wrongplan = 0 if plan == claimsplan ; 
	tab plan claimsplan if wrongplan == 1 ; 
	assert wrongplan != . ; 
	
	egen maxwrong = max(wrongplan), by(familyid) ;
	egen maxcount4 = max(count4), by(familyid) ;
	tab maxwrong wrongplan ; 
	tab maxcount4 count4 ; 
	tab maxwrong maxcount4 ; 
	assert maxcount4 != . ; 
	assert maxwrong != . ;

	replace doublecov = 1 if maxwrong > 0 ; 
	replace doublecov = 1 if maxcount4 > 1 ; 

	drop wrongplan maxwrong maxcount4 ; 
	
	* we don't drop double covered here because of preferences on construction of sample selection table ; 
	* we will drop them and create a row for the table below ; 

* 2 Drop COBRA and self-pay retiree coverage months ; 
	gen empfirst=substr(empid, 1, 1) ;
	* retirees ;
	drop if empfirst=="R" ;
	drop if entitynum=="7777" ;
	* cobra ; 
	drop if empfirst=="B" ;
	tab memid ; 
	drop if memid > 7 ; 
	preserve ; 
		keep if planyear == `year' ; 
		keep personid familyid ; 
		duplicates drop ; 
		gen count_individuals = _N ;
		isid(personid) ;
		drop personid   ; 
		duplicates drop ; 
		isid(familyid) ;
		gen count_family = _N ; 
		drop familyid ;
		duplicates drop ; 
		assert _n == 1 ; 
		gen step = 2 ; 
		gen description = "Number of unique individuals and families not covered by COBRA or self-pay retirees" ; 
		order step description count_individuals count_family ;  
		list ; 
		tempfile step2 ; 
		save `step2';
	restore ;
	
* 3 drop if person switched entities, empids, or memids during planyear ; 
* we assign premium subsidies and choice sets by this set of variables  ; 
* so any change in these variables compromises our ability to accurately ; 
* model an individual's choice ; 
	preserve ; 
		keep if planyear == `year' ;
		keep personid empid memid entitynum tiercd;
		duplicates drop ; 
		gen counter = 1 ; 
		egen dups = sum(counter), by(personid) ;
		tab dups ; 
		drop empid memid entitynum tiercd ; 
		duplicates drop ; 
		tempfile switched ; 
		save `switched' ; 
	restore ; 
	
	merge m:1 personid using `switched' ; 
	tab dups ; 
	tab _merge ; 

	assert _merge == 3 if planyear == `year' ; 
	drop if dups != 1 ; 
	drop dups _merge ; 
	preserve ; 
		keep if planyear == `year' ; 
		keep personid familyid ; 
		duplicates drop ; 
		gen count_individuals = _N ;
		isid(personid) ;
		drop personid ; 
		duplicates drop ; 
		isid(familyid) ;
		gen count_family = _N ; 
		drop familyid ;
		duplicates drop ; 
		assert _n == 1 ; 
		gen step = 3 ; 
		gen description = "Number of unique individuals and families that did not switch entities, employee, member types, or tier mid-year" ; 
		order step description count_individuals count_family ;  
		list ; 
		tempfile step3 ; 
		save `step3' ;
	restore ;
 
* 4 Drop any remaining masked entities ; 
	gen entfirst=substr(entitynum, 1, 1);
	drop if entfirst=="5" ;
	preserve ; 
		keep if planyear == `year' ; 
		keep personid familyid ; 
		duplicates drop ; 
		gen count_individuals = _N ;
		isid(personid) ;
		drop personid ; 
		duplicates drop ; 
		isid(familyid) ;
		gen count_family = _N ; 
		drop familyid ;
		duplicates drop ; 
		assert _n == 1 ; 
		gen step = 4 ; 
		gen description = "Number of unique individuals and families in known entities" ; 
		order step description count_individuals count_family ;  
		list ; 
		tempfile step4 ; 
		save `step4' ;
	restore ;

	preserve ; 
* 5 Require that we have premium and contribution data ; 
	use planyear entitynum empid memid plan medical using `premiumdata', clear ; 
	duplicates drop ; 
	isid(entitynum empid memid plan planyear) ; 
	keep if planyear == `year' ; 
	isid(entitynum empid memid plan);
	drop planyear ; 
	tempfile premium ; 
	save `premium' ; 
	restore ; 
	
	destring entitynum, replace ;
	replace plan = substr(plan,1,5) ;
	merge m:1 entitynum empid memid plan using `premium' ; 
	drop if _merge == 2 ; 

	* Why are there observations in the eligibility file at are not in the contribution survey document?;
	* we should not need this drop, there should be no _merge == 1 as far as I can tell ; 
	* I'm not certain why we don't have these entity*mem*emp observations in the contribution data ; 
	* But across all years and both models, only one of them might stay in the sample anyway based on eligibility requirement ; 
	* so drop them, maybe they were temporary positions so weren't in a master list from OEBB or something. ; 
	drop if _merge == 1 ; 	

	tab medical ; 
	drop if medical == 5 ; 
	drop _merge ; 
	preserve ; 
		keep personid familyid ; 
		duplicates drop ; 
		gen count_individuals = _N ;
		isid(personid) ;
		drop personid ; 
		duplicates drop ; 
		isid(familyid) ;
		gen count_family = _N ; 
		drop familyid ;
		duplicates drop ; 
		assert _n == 1 ; 
		gen step = 5 ; 
		gen description = "Number of unique individuals and families with premium and contribution data" ; 
		order step description count_individuals count_family ;  
		list ; 
		tempfile step5 ; 
		save `step5';
	restore ;

* 6 Require at least 2 choices ; 
	preserve ; 
	use `premiumdata', clear ; 
	keep if planyear == `year' ; 
	drop if medical == 5 ; 
	keep entitynum memid empid plan ;
	duplicates drop ; 
	gen counter = 1 ; 
	egen choices = sum(counter), by(entitynum memid empid) ;
	drop plan ; 
	duplicates drop ; 
	drop counter ; 
	tab choices ; 
	di "Year is `year'" ; 

	tempfile choices ; 
	save `choices' ; 
	restore ; 
	merge m:1 entitynum empid memid using `choices' ; 
	assert _merge != 1 ; 
	drop if _merge == 2 ; 
	drop _merge ; 
	tab choices ; 
	drop if choices == 1 ; 
		
	preserve ; 
		keep if planyear == `year' ; 
		keep personid familyid ; 
		duplicates drop ; 
		gen count_individuals = _N ;
		isid(personid) ;
		drop personid ; 
		duplicates drop ; 
		isid(familyid) ;
		gen count_family = _N ; 
		drop familyid ;
		duplicates drop ; 
		assert _n == 1 ; 
		gen step = 6 ; 
		gen description = "Number of unique individuals and families with at least 2 choices" ; 
		order step description count_individuals count_family ;  
		list ; 
		tempfile step6 ; 
		save `step6';
	restore ;

* 7 Impose continuous eligibility requirements ; 
	* Individuals must be in the eligibility file, and must be covered by a subscriber in the eligibily file ; 
	* For the full year for PF ; 
	gen month = month(eligmo_date) ;
	gen year = year(eligmo_date) ;
	* make sure year month personid is unique ; 
	isid (personid month year) ; 
	
	drop counter ; 
	gen counter = 1 ; 
	egen months_elig = sum(counter), by (personid) ; 
	tab months_elig ;
	
	*For PF we want to keep new beneficiaries, so do not require pre-enrollment eligibility ; 
	keep if months_elig == 12 ;
	drop months_elig ; 
	
	* check that everyone who is continuously eligible is covered by a continously eligible subscriber ; 
	cap drop member ; 
	cap drop subscriber ; 
	gen subscriber=substr(personid, 1, 9) ; 
	gen member=substr(personid, 15, 23) ; 
	preserve ; 
		keep if subscriber == member ; 
		keep familyid ; 
		duplicates drop ; 
		tempfile famelig ; 
		save `famelig' ; 
	restore ; 
	merge m:1 familyid using `famelig' ; 

	keep if _merge == 3 ; 
	drop _merge ; 
	preserve ; 
		keep if planyear == `year' ; 
		keep personid familyid ; 
		duplicates drop ; 
		gen count_individuals = _N ;
		isid(personid) ;
		drop personid ; 
		duplicates drop ; 
		isid(familyid) ;
		gen count_family = _N ; 
		drop familyid ;
		duplicates drop ; 
		assert _n == 1 ; 
		gen step = 7 ; 
		gen description = "Number of unique individuals and families with continous eligibility for ONE YEAR ONLY" ; 
		order step description count_individuals count_family ;  
		list ; 
		tempfile step7 ; 
		save `step7';
	restore ;
	
	* not certain this belongs here now ; 
	* one observation per person - keep beginning of coverage month ; 
	gen october = 1 if month == 10 ; 
	replace october = 0 if month != 10 ; 
	egen test = max(october), by(personid); 
	assert test == 1; 
	drop test october ; 
	keep if month==10 ; 
	isid(personid) ; 
	isid(personidIE) ; 
		
* 8 Now drop double covered ; 
	tab doublecov ; 
	replace doublecov = 0 if doublecov == . ; 
	tab doublecov ; 
	egen maxdouble = max(doublecov), by(familyid) ; 
	tab maxdouble ; 
	drop if maxdouble == 1; 
	assert doublecov == 0 ; 
	preserve ; 
		keep if planyear == `year' ; 
		keep personid familyid ; 
		duplicates drop ; 
		gen count_individuals = _N ;
		isid(personid) ;
		drop personid ; 
		duplicates drop ; 
		isid(familyid) ;
		gen count_family = _N ; 
		drop familyid ;
		duplicates drop ; 
		assert _n == 1 ; 
		gen step = 8 ; 
		gen description = "Number of unique individuals and families covered by only one subscriber and with claims in only one plan" ; 
		order step description count_individuals count_family ;  
		list ; 
		tempfile step8 ; 
		save `step8';
	restore ;
	
* 9 make sure we drop the entire family if we have dropped any members ; 
	merge m:1 familyid using `famsize' ; 
	unique(personid) ;
	drop if _merge == 2 ; 
	drop _merge ; 
	
	drop counter ; 
	gen counter = 1 ; 
	egen size = sum(counter), by(familyid) ; 
	tab size famsize ; 
	drop if size == 1 & tier !="EMP" ;
	tab famsize size ; 

	drop if size !=famsize ;
	
	* Now make sure we still have the policy holder ; 
	gen holder = 1 if subscriber == member ;
	replace holder = 0 if subscriber != member ;
	egen maxholder = max(holder), by(familyid) ;
	tab maxholder ; 
	drop if maxholder == 0 ; 
	drop holder maxholder ; 

	preserve ; 
		keep if planyear == `year' ; 
		keep personid familyid  ; 
		duplicates drop ; 
		gen count_individuals = _N ;
		isid(personid) ;
		drop personid  ; 
		duplicates drop ; 
		isid(familyid) ;
		gen count_family = _N ; 
		drop familyid ;
		duplicates drop ; 
		assert _n == 1 ; 
		gen step = 9 ; 
		gen description = "Number of unique individuals and families in intact families" ; 
		order step description count_individuals count_family ;  
		list ; 
		tempfile step9 ; 
		save `step9';
	restore ;

	assert medcov == 1 ; 
	assert statusgrp == 1 ; 
	drop statusgrp ; 

	* retiress / cobra check ; 
	assert memid < 8 ; 
	
	* Not explicitly sample selection: Some data cleaning and preparation for later programs ; 
	bys personid: egen eligstrt = min(eligmo_date) ;
	format eligstrt %d ;
	gen planname = substr(plan, 1, 5);
	gen planfirst=substr(planname,1,1);
	gen drugplanname = substr(drugplan, 1, 4);
	replace drugplanname = trim(drugplanname) ; 
	
	* Confirm this manipulation of drug plans ; 
	if `year'==2012 { ;
	replace drugplanname ="ORXB" if planfirst=="O" & planname!="OMED9"; 
	// In 2012 there were no longer three diffeerent drug plans to choose from for ODS or 2 for Kaiser.  ;
	replace drugplanname ="KRX1" if planfirst=="K";
	} ;

	if `year'==2013 { ;
	replace drugplanname = "MMED" if planfirst=="M" & planname != "MMEDH" ;
	replace drugplanname = "MMEDH" if planname == "MMEDH" ;
	replace drugplanname = "KRX1" if planfirst=="K";
	} ;
		
	if `year' < 2013 { ;
	replace drugplanname="OMED9" if drugplanname=="NE" & planname=="OMED9" & planfirst == "O";
	assert planfirst != "M" ; 
	
	replace drugplanname="ORXB" if drugplanname=="ORXA" & planyear>2011 & planfirst == "O"; 
	// We only want one name for 2012 - this is more or less a failsafe.  ;
	replace drugplanname="ORXB" if drugplanname=="NE" & planyear>2011 & planfirst == "O"; 
	// Same as above.  ;

	gen tag=1 if drugplanname =="NE";
	replace tag=0 if tag==.;
	bys personid: egen mintag=min(tag); 
	// We want to make sure that no personid has all bad drugplan names.  ;
	assert mintag==0; 
	drop if tag==1; drop mintag tag; 	
	} ;
	
	drop if medcov==0;
	rename planname chosenplan;
	rename drugplanname chosendrgplan;
	
	* Generate and check tier values ; 
	gen tier = 1 if tiercd == "EMP"; // employee;
	replace tier = 2 if tiercd == "ESP"; // employee spouse;
	replace tier = 3 if tiercd == "ECH"; // employee child;
	replace tier = 4 if tiercd == "FAM"; // family;

	keep personid personidIE eligmo_date planyear familyid entitynum chosenplan chosendrgplan eligstrt tier memid empid sex age_yr ; 
	duplicates drop;
	
	* Let's inspect the tier variable, and then give everybody in the same family the same tier variable value.  ;
	* this seems like a good idea, but only effects 2 observations - what's unique about them?; 
	bys familyid: egen maxtier=max(tier);
	gen tag1=1 if maxtier !=tier;
	egen tag=count(tag1);
	assert tag <10;
	count if maxtier !=tier;
	replace tier=maxtier if maxtier !=tier;
	drop tag tag1 maxtier;
	
	tab chosendrgplan chosenplan ; 
	
	sort entitynum empid memid familyid personid ; 
	
	preserve ; 
		use `med0', clear ; 
		keep familyid personid ; 
		duplicates drop ; 
		tempfile med1 ; 
		save `med1'  ;
		
		use `drug0', clear ; 
		keep familyid personid ; 
		duplicates drop ; 
		tempfile drug1 ; 
		save `drug1'  ;
	restore ; 	

	merge 1:1 personid familyid using `med1' ; 
	* in sample and with med claims ; 
	gen anymed = 1 if _merge == 3 ; 
	* not in sample, but had claims ; 
	drop if _merge == 2 ; 
	* in sample, but not med claims ; 
	replace anymed = 0 if _merge == 1 ; 
	assert anymed != . ; 
		
	drop _merge ; 
	merge 1:1 personid familyid using `drug1' ; 
	gen anydrug = 1 if _merge == 3 ; 
	drop if _merge == 2 ; 
	replace anydrug = 0 if _merge == 1 ; 
	assert anydrug != . ; 
	save "`out_data'/sample_PF_`year'_`cdate'.dta", replace ; 
	
	* we are just making a new table for the paper ; 
	use `step0', clear ; 
		append using `step1' ; 
		append using `step2' ; 
		append using `step3' ; 
		append using `step4' ; 
		append using `step5' ; 
		append using `step6' ;
		append using `step7' ;
		append using `step8' ;
		append using `step9' ;
	list ;	
clear ;
} ;
capture log close ; 